"""
--------------------------------------------------------------------------
OSD335x C-SiP sensor
--------------------------------------------------------------------------
Copyright 2019, Octavo Systems, LLC. All rights reserved.

License:     
Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, 
this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, 
this list of conditions and the following disclaimer in the documentation 
and/or other materials provided with the distribution.

3. Neither the name of the copyright holder nor the names of its 
contributors may be used to endorse or promote products derived from 
this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE 
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
POSSIBILITY OF SUCH DAMAGE.
--------------------------------------------------------------------------
OSD335x C-SiP sensor

    The program will measure and transmit sensor readings from various sensors on board the OSD335x C-SiP sensor board

--------------------------------------------------------------------------
Software Setup:

    * Use application note: https://octavosystems.com/app_notes/programming-emmc-with-usb/
    * Libraries used:
      https://github.com/adafruit/Adafruit_Python_BNO055
      https://github.com/adafruit/Adafruit_Python_VCNL40xx
--------------------------------------------------------------------------
"""
import time
import os
import logging
import sys
import multiprocessing

# Import the VCNL40xx module.
import Adafruit_VCNL40xx
from Adafruit_BNO055 import BNO055

# ------------------------------------------------------------------------------
# Cosntants
# ------------------------------------------------------------------------------
gw_ip_address    = "192.168.8.1"
gw_port          = "10000"
MSG_CLIENT       = "/var/lib/cloud9/gw_demo/msg_client"

# Create a VCNL4010 instance.
vcnl = Adafruit_VCNL40xx.VCNL4010(busnum=0)
bno = BNO055.BNO055(busnum=2, rst='P9_12')

if len(sys.argv) == 2 and sys.argv[1].lower() == '-v':
    logging.basicConfig(level=logging.DEBUG)

# Initialize the BNO055 and stop if something went wrong.
if not bno.begin():
    raise RuntimeError('Failed to initialize BNO055! Is the sensor connected?')

# Print system status and self test result.
status, self_test, error = bno.get_system_status()
print('System status: {0}'.format(status))
print('Self test result (0x0F is normal): 0x{0:02X}'.format(self_test))
# Print out an error if system status is in error mode.
if status == 0x01:
    print('System error: {0}'.format(error))
    print('See datasheet section 4.3.59 for the meaning.')

# Print BNO055 software revision and other diagnostic data.
sw, bl, accel, mag, gyro = bno.get_revision()
print('Software version:   {0}'.format(sw))
print('Bootloader version: {0}'.format(bl))
print('Accelerometer ID:   0x{0:02X}'.format(accel))
print('Magnetometer ID:    0x{0:02X}'.format(mag))
print('Gyroscope ID:       0x{0:02X}\n'.format(gyro))

def transmit_data(results):

  transmit_command = "{0} {1} {2}".format(MSG_CLIENT, gw_ip_address, gw_port)
  try:
    os.system("printf \"{0}\\n\" | {1}".format(results, transmit_command))
  except:
    print("Cannot transmit results!")

def send_update(results):
  '''This funcion will periodically send sensor data to gateway'''
  p = multiprocessing.Process(target=transmit_data, args=(results,))
  p.start()
  # Wait for timeout or until thread finishes
  p.join(timeout)
  if p.is_alive():
    print("Cannot transmit results!")
    p.terminate()
    p.join()


print('Printing proximity data (press Ctrl-C to quit)...')
timeout = 5.0
print("Starting C-SiP sensor")
print("| Proximity  |     Ambient     |    Heading   |      Roll      |      Pitch     |")
print("|------------|-----------------|--------------|----------------|----------------|")

while True:
    # Read proximity.
    proximity = vcnl.read_proximity()
    # Read the ambient light.
    ambient = vcnl.read_ambient()
    # Read gyro values
    heading, roll, pitch = bno.read_euler()
    results = "CS {0} {1} {2:0.2F} {3:0.2F} {4:0.2F}".format(proximity, ambient, heading, roll, pitch)
    print("| {:10d} | {:15d} | {:12.2f} | {:14.2f} | {:14.2f} |".format(proximity, ambient, heading, roll, pitch))
    send_update(results)
    time.sleep(0.3)
