/*
 * @file   client_socket.cc
 * @author David Weber <david.weber@octavosystems.com>
 * @date   02/14/2019
 *
 * @brief client_socket module implementation
 *
 * Copyright © 2019 Octavo Systems, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <stdio.h>
#include <string.h>
#include <netdb.h>
#include <fcntl.h>
#include "socket.hh"
#include "client_socket.hh"
#include "rtstatus.h"

//-----------------------------------------------------------------------------

client_socket::client_socket(char* ip_string,    // dotted-decimal notation
                             unsigned short port, 
                             unsigned short addr_family, 
                             int protocol_family, 
                             int type, 
                             int protocol) : socket(protocol_family, 
                                                    type, protocol)
{
  this->ip_string = new char[strlen(ip_string) + 1];
  if (this->ip_string != NULL)
  {
    strcpy(this->ip_string, ip_string);
  }
  memset(&this->addr, 0, sizeof(this->addr));
  this->addr.sin_family = addr_family;
  this->addr.sin_port = htons(port);  
}

//-----------------------------------------------------------------------------

client_socket::~client_socket()
{
  if (ip_string != NULL)
  {
    delete(ip_string);
  }
}

//-----------------------------------------------------------------------------

rt_status_t client_socket::connect()
{
  rt_status_t status = socket::open();
  
  if (status == SUCCESS)
  {
    struct hostent* hp = gethostbyname(this->ip_string);
    memcpy((char*)&addr.sin_addr.s_addr, (char*)hp->h_addr, hp->h_length);
    if (::connect(socket_handle, (struct sockaddr*)&addr, sizeof(addr)) < 0)
    {
      status = SOCKET_CONNECT_FAILURE;
    } 
  }
  return status;
}

//------------------------------------------------------------------------------

rt_status_t client_socket::connect_non_block()
{
  fcntl(socket_handle, F_SETFL, fcntl(socket_handle, F_GETFL, NULL) | O_NONBLOCK);
  return this->connect();
}

//-----------------------------------------------------------------------------

ssize_t client_socket::recv(void* recv_data, ssize_t recv_len)
{
  return socket::recv(socket_handle, recv_data, recv_len);
}

//-----------------------------------------------------------------------------

ssize_t client_socket::send(void* send_data, ssize_t send_len)
{
  return socket::send(socket_handle, send_data, send_len);
}

//-----------------------------------------------------------------------------

rt_status_t client_socket::recv_wait(void* recv_data, ssize_t recv_len)
{
  return socket::recv_wait(socket_handle, recv_data, recv_len);
}

//-----------------------------------------------------------------------------

rt_status_t client_socket::send_wait(void* send_data, ssize_t send_len)
{
  return socket::send_wait(socket_handle, send_data, send_len);
}

//-----------------------------------------------------------------------------

