/*
 * @file   server_socket.cc
 * @author David Weber <david.weber@octavosystems.com>
 * @date   02/14/2019
 *
 * @brief server_socket module implementation
 *
 * Copyright © 2019 Octavo Systems, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include "server_socket.hh"
#include "rtstatus.h"

//-----------------------------------------------------------------------------

server_socket::server_socket(unsigned short port, 
                             unsigned long addr, 
                             unsigned short addr_family, 
                             int protocol_family, 
                             int type, 
                             int protocol) : socket(protocol_family, 
                                                    type, protocol)
{
  client_handle = -1;
  memset(&this->addr, 0, sizeof(addr));
  this->addr.sin_family = addr_family;
  this->addr.sin_addr.s_addr = htonl(addr);
  this->addr.sin_port = htons(port);  
}

//-----------------------------------------------------------------------------

server_socket::~server_socket()
{
  server_socket::close();
}

//-----------------------------------------------------------------------------

rt_status_t server_socket::open()
{
  rt_status_t status = SUCCESS;

  status = socket::open();

  if (status == SUCCESS)
  {
    if (bind(socket_handle, (struct sockaddr*)&addr, sizeof(addr)) < 0)
    {
      status = SOCKET_BIND_FAILURE;
    } 
    else if (listen(socket_handle, 1) < 0)
    {
      status = SOCKET_LISTEN_FAILURE;
    }  
    if (status != SUCCESS)
    {
      socket::close();
    }
  }

  return status;
}

//-----------------------------------------------------------------------------

rt_status_t server_socket::wait_for_connection()
{
  sockaddr_in_t client_addr;
  rt_status_t status = SUCCESS;

  socklen_t client_addr_len = sizeof(sockaddr_in_t);

  client_handle = accept(socket_handle, (struct sockaddr*)&client_addr, 
                         &client_addr_len);

  if (client_handle < 0)
  {
    status = SOCKET_ACCEPT_FAILURE;
  }
  return status;
}

//-----------------------------------------------------------------------------

void server_socket::close()
{
  socket::close();
}

//-----------------------------------------------------------------------------

void server_socket::close_client_handle()
{
  ::close(client_handle);
}

//-----------------------------------------------------------------------------

ssize_t server_socket::recv(void* recv_data, unsigned int recv_len)
{
  return socket::recv(client_handle, recv_data, recv_len);
}

//-----------------------------------------------------------------------------

ssize_t server_socket::send(void* send_data, unsigned int send_len)
{
  return socket::send(client_handle, send_data, send_len);
}

//-----------------------------------------------------------------------------

rt_status_t server_socket::recv_wait(void* recv_data, unsigned int recv_len)
{
  return socket::recv_wait(client_handle, recv_data, recv_len);
}

//-----------------------------------------------------------------------------

rt_status_t server_socket::send_wait(void* send_data, unsigned int send_len)
{
  return socket::send_wait(client_handle, send_data, send_len);
}

//-----------------------------------------------------------------------------

