/*
 * @file   socket.cc
 * @author David Weber <david.weber@octavosystems.com>
 * @date   02/14/2019
 *
 * @brief socket module implementation
 *
 * Copyright © 2019 Octavo Systems, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <signal.h>
#include <errno.h>
#include <unistd.h>
#include "rtstatus.h"
#include "socket.hh"

//-----------------------------------------------------------------------------

socket::socket(int protocol_family, int type, int protocol)
{
  this->socket_handle = 0;
  this->protocol_family = protocol_family;
  this->type = type;
  this->protocol = protocol;
  this->socket_opened = false;
}

//-----------------------------------------------------------------------------

socket::~socket()
{
  close();
}

//-----------------------------------------------------------------------------

void socket::close()
{
  if (socket_handle > 0)
  {
    ::shutdown(socket_handle, SHUT_RDWR);
    ::close(socket_handle);
    socket_handle = 0;
    socket_opened = false;
  }
}

//-----------------------------------------------------------------------------

rt_status_t socket::open()
{
  rt_status_t status = SUCCESS;

  if (!socket_opened)
  {
    socket_handle = ::socket(protocol_family, type, protocol);
    if (socket_handle == -1)
    {
      status = SOCKET_OPEN_FAILURE;
    }
    else
    {
      socket_opened = true;
    }
  } 
  return status;
}

//-----------------------------------------------------------------------------

ssize_t socket::send(int sockid, void* buf, unsigned int len, int flags)
{
  ssize_t sent = ::send(sockid, buf, len, flags);
  return sent;
}

//-----------------------------------------------------------------------------

ssize_t socket::recv(int sockid, void* buf, unsigned int len, int flags)
{
  ssize_t received = ::recv(sockid, buf, len, flags);
  return received;
}

//-----------------------------------------------------------------------------

rt_status_t socket::send_wait(int sockid, void* buf, unsigned int len, int flags)
{
  rt_status_t status = SUCCESS;
  ssize_t sent = 0;
  ssize_t total_sent = 0;
  uint8_t* u8buf = (uint8_t*)buf;

  do
  {
    sent = ::send(sockid, &u8buf[total_sent], len - total_sent, flags);
    if (sent > 0)
    {
      total_sent += sent;
    }
    else if (sent < 0)
    {
      status = SOCKET_SEND_FAILURE;
    }
  } while ((total_sent != len) && (status == SUCCESS));

  return status;
}

//-----------------------------------------------------------------------------

rt_status_t socket::recv_wait(int sockid, void* buf, unsigned int len, int flags)
{
  rt_status_t status = SUCCESS;
  ssize_t received = 0;
  ssize_t total_received = 0;
  uint8_t* u8buf = (uint8_t*)buf;

  do
  {
    received = ::recv(sockid, &u8buf[total_received], len - total_received, flags);
    if (received > 0)
    {
      total_received += received;
    }
    else if (received == 0)
    {
      status = SOCKET_DISCONNECT_DETECTED;
    }
    else if (received < 0)
    {
      if (errno == ECONNRESET)
      {
        status = SOCKET_DISCONNECT_DETECTED;
      }
      else
      {
        status = SOCKET_RECV_FAILURE;
      }
    }
  } while ((total_received < len) && (status == SUCCESS));

  return status;   
}

